(defstruct token
  (type nil :type (or null (member string attr size color)))
  (val nil :type (or string symbol fixnum character))
  (len 0 :type fixnum))

(defun make-token-table ()
  (let ((ret (make-array '(256 256) :initial-element nil
			 :element-type '(or null token))))
    (dolist (spec '(("B" bold 1)
		    ("EM" emph 2)
		    ("I" italic 1)
		    ("PL" plain 2)
		    ("S" strong 1)
		    ("TT" tt 2)
		    ("U" uline 1)))
      (let ((fst (char (car spec) 0))
	    (snd (if (= (length (car spec)) 1)
		     #\>
		   (char (car spec) 1))))
	(setf (aref ret (char-code fst) (char-code snd))
	      (make-token :type 'attr :val (cadr spec) :len (caddr spec)))))
    (dotimes (i 10)
      (let ((fst (+ i (char-code #\0)))
	    (snd (char-code #\>)))
	(setf (aref ret fst snd)
	      (make-token :type 'size :val i :len 1))))
    (dolist (spec '("r"
		    "g"
		    "b"
		    "c"
		    "m"
		    "y"
		    "k"
		    "w"))
      (let ((fst (char-code (char spec 0)))
	    (snd (char-code #\>)))
	(setf (aref ret fst snd)
	      (make-token :type 'color :val (char spec 0) :len 1))))
    ret))

(defun make-space-tab ()
  (let ((ret (make-string 256 :initial-element #\Space)))
    (dotimes (i 256)
      (let ((c (code-char i)))
	(case c
	  ((#\Space #\Linefeed #\Return #\Tab) nil)
	  (otherwise (setf (schar ret i) c)))))
    ret))
